//
//  MainMenuLayer.m
//  CowboyGame
//
//  Created by Michael on 14/02/2012.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "MainMenuLayer.h"

@interface MainMenuLayer()
-(void)displayMainMenu;
-(void)displayMainMenuStats;
-(void)displayOptionsMenu;
-(void)displayStatsMenu;

@end

@implementation MainMenuLayer

#pragma mark -
#pragma mark Functions Called on Menus
-(void)playGame {
    CCLOG(@"Show the Card Select Screen");
    [[GameManager sharedGameManager] runSceneWithID:kCardSelect];
}

-(void)toggleMusic {
    CCLOG(@"toggling music");
    if ([[GameManager sharedGameManager] isMusicOn]) {
		CCLOG(@"OptionsLayer-> Turning Game Music OFF");
		[[GameManager sharedGameManager] setIsMusicOn:NO];
	    if([[SimpleAudioEngine sharedEngine]isBackgroundMusicPlaying])
        {
            [[SimpleAudioEngine sharedEngine] pauseBackgroundMusic];
        } else {
            [[SimpleAudioEngine sharedEngine] resumeBackgroundMusic];
        }
    } else {
		CCLOG(@"OptionsLayer-> Turning Game Music ON");
		[[GameManager sharedGameManager] setIsMusicOn:YES];
	}

}

-(void)doNothing {
    CCLOG(@"THIS DON'T DO STUFF");
}

-(void) clearStats {
    CCLOG(@"Clearing Stats");
    NSError *error;
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES); //1
    
    NSString *documentsDirectory = [paths objectAtIndex:0]; //2
    
    NSString *path = [documentsDirectory stringByAppendingPathComponent:@"Stats.plist"]; //3  
    
    NSFileManager *fileManager = [NSFileManager defaultManager];  
    
    if (![fileManager fileExistsAtPath: path]) //4
    {
        NSString *bundle = [[NSBundle mainBundle] pathForResource:(@"Stats") ofType:(@"plist")]; //5
        
        [fileManager copyItemAtPath:bundle toPath: path error:&error]; //6
    }
    
    NSMutableDictionary *data = [[NSMutableDictionary alloc] initWithContentsOfFile: path];
    
    // Add 1 to games play 
    [data setObject:[NSNumber numberWithInt:0] forKey:@"MatchesPlayed"];
    [data setObject:[NSNumber numberWithInt:0] forKey:@"MatchesWon"];
    [data setObject:[NSNumber numberWithInt:0] forKey:@"MatchesLost"];
    [data writeToFile: path atomically:YES];
    [data release];
}

#pragma mark -
#pragma mark Menus
-(void)displayMainMenu {
    CCLOG(@"Showing Main Menu");
    CGSize screenSize = [CCDirector sharedDirector].winSize;
    if (optionsMenu != nil) {
        [optionsMenu removeFromParentAndCleanup:YES];
    }
    
    // Create Title
    CCLabelTTF *title = [CCLabelTTF labelWithString:@"Legends of the West" fontName:@"American Typewriter" fontSize:45];
    title.position = ccp(240,275);
    title.color = ccBLACK;
    [self addChild:title];
    
    // Setup Main Menu Items
    CCMenuItem *play = [CCMenuItemFont itemFromString:@"Play" target:self selector:@selector(playGame)];
    [(CCMenuItemFont *)play setColor:ccBLACK];
    CCMenuItem *options = [CCMenuItemFont itemFromString:@"Options" target:self selector:@selector(displayOptionsMenu)];
    [(CCMenuItemFont *)options setColor:ccBLACK];
    CCMenuItem *stats = [CCMenuItemFont itemFromString:@"My Stats" target:self selector:@selector(displayStatsMenu)];
    [(CCMenuItemFont *)stats setColor:ccBLACK];
    
    mainMenu = [CCMenu menuWithItems:play,options,stats, nil];
    [mainMenu alignItemsVerticallyWithPadding:screenSize.height * 0.059f];
    [mainMenu setPosition:ccp(240,160)];
    [self addChild:mainMenu z:0 tag:kMainMenuTagValue];

}

-(void)displayMainMenuStats {
    CCLOG(@"Showing Main Menu from Stats Menu");
    CGSize screenSize = [CCDirector sharedDirector].winSize;
    if (statsMenu != nil) {
        [statsMenu removeFromParentAndCleanup:YES];
    }
    
    // Create Title
    CCLabelTTF *title = [CCLabelTTF labelWithString:@"Legends of the West" fontName:@"American Typewriter" fontSize:45];
    title.position = ccp(240,275);
    title.color = ccBLACK;
    [self addChild:title];
    
    // Setup Main Menu Items
    CCMenuItem *play = [CCMenuItemFont itemFromString:@"Play" target:self selector:@selector(playGame)];
    [(CCMenuItemFont *)play setColor:ccBLACK];
    CCMenuItem *options = [CCMenuItemFont itemFromString:@"Options" target:self selector:@selector(displayOptionsMenu)];
    [(CCMenuItemFont *)options setColor:ccBLACK];
    CCMenuItem *stats = [CCMenuItemFont itemFromString:@"My Stats" target:self selector:@selector(displayStatsMenu)];
    [(CCMenuItemFont *)stats setColor:ccBLACK];
    
    mainMenu = [CCMenu menuWithItems:play,options,stats, nil];
    [mainMenu alignItemsVerticallyWithPadding:screenSize.height * 0.059f];
    [mainMenu setPosition:ccp(240,160)];
    [self addChild:mainMenu z:0 tag:kMainMenuTagValue];
    
}

-(void)displayOptionsMenu {
    CCLOG(@"Showing Main Menu");
    CGSize screenSize = [CCDirector sharedDirector].winSize;
    if (mainMenu !=nil) {
        [mainMenu removeFromParentAndCleanup:YES];
    }
    
    // Setup Options Menu
    CCMenuItem *toggleMusic = [CCMenuItemFont itemFromString:@"Toggle Background Music" target:self selector:@selector(toggleMusic)];
    [(CCMenuItemFont *)toggleMusic setColor:ccBLACK];
    CCMenuItem *clearStats = [CCMenuItemFont itemFromString:@"Clear Stats" target:self selector:@selector(clearStats)];
    [(CCMenuItemFont *)clearStats setColor:ccBLACK];
    CCMenuItem *backToMenu = [CCMenuItemFont itemFromString:@"Back To Menu" target:self selector:@selector(displayMainMenu)];
    [(CCMenuItemFont *)backToMenu setColor:ccBLACK];

    optionsMenu = [CCMenu menuWithItems:toggleMusic,clearStats,backToMenu, nil];
    [optionsMenu alignItemsVerticallyWithPadding:screenSize.height * 0.059f];
    [optionsMenu setPosition:ccp(240,160)];
    [self addChild:optionsMenu z:1 tag:kOptionsTagValue];
    

}

-(void)displayStatsMenu {
    CCLOG(@"Showing Stats Menu");
    CGSize screenSize = [CCDirector sharedDirector].winSize;
    if (mainMenu !=nil) {
        [mainMenu removeFromParentAndCleanup:YES];
    }
    
    
    NSError *error;
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES); //1
    NSString *documentsDirectory = [paths objectAtIndex:0]; //2
    NSString *path = [documentsDirectory stringByAppendingPathComponent:@"Stats.plist"]; //3
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    if (![fileManager fileExistsAtPath: path]) //4
    {
        NSString *bundle = [[NSBundle mainBundle] pathForResource:(@"Stats") ofType:(@"plist")]; //5
        
        [fileManager copyItemAtPath:bundle toPath: path error:&error]; //6
    }
    
    NSMutableDictionary *savedStock = [[NSMutableDictionary alloc] initWithContentsOfFile: path];
    
    //load from savedStock
    int gamesPlayedValue;
    int gamesWonValue;
    int gamesLostValue;
    
    gamesPlayedValue = [[savedStock objectForKey:@"MatchesPlayed"] intValue];
    NSString* gamesPlayedString = [NSString stringWithFormat:@"Games Played: %d", gamesPlayedValue];
    gamesWonValue = [[savedStock objectForKey:@"MatchesWon"] intValue];
    NSString* gamesWonString = [NSString stringWithFormat:@"Games Won: %d", gamesWonValue];
    gamesLostValue = [[savedStock objectForKey:@"MatchesLost"] intValue];
    NSString* gamesLostString = [NSString stringWithFormat:@"Games Lost: %d", gamesLostValue];
    
    [savedStock release];
    
    CCMenuItem *statsGP = [CCMenuItemFont itemFromString:gamesPlayedString target:self selector:@selector(doNothing)];
    CCMenuItem *statsGW = [CCMenuItemFont itemFromString:gamesWonString target:self selector:@selector(doNothing)];
    CCMenuItem *statsGL = [CCMenuItemFont itemFromString:gamesLostString target:self selector:@selector(doNothing)];
    CCMenuItem *backToMenu = [CCMenuItemFont itemFromString:@"Back To Menu" target:self selector:@selector(displayMainMenuStats)];
    [(CCMenuItemFont *)statsGP setColor:ccBLACK];
    [(CCMenuItemFont *)statsGW setColor:ccBLACK];
    [(CCMenuItemFont *)statsGL setColor:ccBLACK];
    [(CCMenuItemFont *)backToMenu setColor:ccBLACK];
    
    
    statsMenu = [CCMenu menuWithItems:statsGP,statsGW,statsGL,backToMenu, nil];
    [statsMenu alignItemsVerticallyWithPadding:screenSize.height * 0.059f];
    [statsMenu setPosition:ccp(240,160)];
    [self addChild:statsMenu z:2 tag:kStatsTagValue];

}

-(id)init {
    self = [super init];
    if (self != nil) {
        [CCMenuItemFont setFontSize:30];
		[CCMenuItemFont setFontName: @"American Typewriter"];
        
//        CGSize screenSize = [CCDirector sharedDirector].winSize;
        CCSprite *backgroundImage;
        backgroundImage = [CCSprite spriteWithFile:@"background.png"];
        [backgroundImage setPosition: CGPointMake(240,160)];
        [self addChild:backgroundImage];
        [self displayMainMenu];
        if ([[GameManager sharedGameManager] isMusicOn]) {
            [[GameManager sharedGameManager] playBackgroundTrack:BACKGROUND_TRACK_MAIN_MENU];
        }
        
//        [self loadAudio];
    }
    return self;
}

@end
