//
//  ReadyUpLayer.m
//  CowboyGame
//
//  Created by Michael on 03/02/2012.
//  Copyright 2012 __MyCompanyName__. All rights reserved.
//
#import "ReadyUpLayer.h"

@interface ReadyUpLayer()
-(void)displayReadyUp;
-(void)displayReadiedUp;
-(void)displayResults;

@end

@implementation ReadyUpLayer
@synthesize gameSession;

@synthesize serverPlayerAttack,serverPlayerDefence,serverPlayerFame,serverPlayerTotalValue,serverPlayerRandomValue;

@synthesize clientPlayerAttack,clientPlayerDefence,clientPlayerFame,clientPlayerTotalValue,clientPlayerRandomValue;

-(id)init {
    self = [super init];
    if (self != nil) {
        
        [CCMenuItemFont setFontSize:30];
		[CCMenuItemFont setFontName: @"American Typewriter"];
        
        CCSprite *backgroundImage;
        backgroundImage = [CCSprite spriteWithFile:@"background.png"];
        [backgroundImage setPosition: CGPointMake(240,160)];
        [self addChild:backgroundImage];
        [self displayReadyUp];
    
        peerPicker = [[GKPeerPickerController alloc] init];
        peerPicker.delegate = self;
        
        //There are 2 modes of connection type
        // - GKPeerPickerConnectionTypeNearby via BlueTooth
        // - GKPeerPickerConnectionTypeOnline via Internet
        // We will use Bluetooth Connectivity for this example
        
        peerPicker.connectionTypesMask = GKPeerPickerConnectionTypeNearby;
        peerArray=[[NSMutableArray alloc] init];
    }
    return self;
}



-(void)showMainMenu {
    CCLOG(@"Back to Menu to start again");
    [[GameManager sharedGameManager] runSceneWithID:kMainMenuScene];
}

-(void)doNothing {
    CCLOG(@"THIS DOESN'T DO ANYTHING");
}


#pragma mark -
#pragma mark Layers
-(void)displayReadyUp {
    CCLOG(@"Displaying Ready Up");
    CGSize screenSize = [CCDirector sharedDirector].winSize;
    
    
    // Setup Ready Up Items
    //for testing, allow click
    CCMenuItem *instructions = [CCMenuItemFont itemFromString:@"Fast Draw to Ready Up!" target:self selector:@selector(doNothing)];
    [(CCMenuItemFont *)instructions setColor:ccBLACK];
    
    readyUp = [CCMenu menuWithItems:instructions, nil];
    [readyUp alignItemsVerticallyWithPadding:screenSize.height * 0.059f];
    [readyUp setPosition:ccp(240,160)];
    [self addChild:readyUp z:0 tag:kReadyUpTagValue];
    
    [[UIAccelerometer sharedAccelerometer] setUpdateInterval:0.80] ;
    [[UIAccelerometer sharedAccelerometer] setDelegate:self];

}

-(void)displayReadiedUp {
    CCLOG(@"Displaying Readied Up Menu");
    CGSize screenSize = [CCDirector sharedDirector].winSize;
    if (readyUp != nil) {
        [readyUp removeFromParentAndCleanup:YES];
    }
    
    // Kill the accelerometer
    [[UIAccelerometer sharedAccelerometer] setDelegate:nil];
    
    // Setup Readied up
    CCMenuItem *readyToGo = [CCMenuItemFont itemFromString:@"Waiting for Connection..." target:self selector:@selector(displayResults)];
    [(CCMenuItemFont *)readyToGo setColor:ccBLACK];
    CCMenuItem *backToMenu = [CCMenuItemFont itemFromString:@"Back" target:self selector:@selector(showMainMenu)];
    [(CCMenuItemFont *)backToMenu setColor:ccBLACK];
    
    readiedUp = [CCMenu menuWithItems:readyToGo, backToMenu, nil];
    [readiedUp alignItemsVerticallyWithPadding:screenSize.height * 0.075f];
    [readiedUp setPosition:ccp(240,160)];
    [self addChild:readiedUp z:0 tag:kReadiedUpTagValue];
    
    //run grab connection code
    [self startGameplay];
}

-(void)displayResults {
    CCLOG(@"Displaying Results");
    CGSize screenSize = [CCDirector sharedDirector].winSize;
    if (readiedUp !=nil) {
        [readiedUp removeFromParentAndCleanup:YES];
    }
    
    NSString* outputString = [NSString stringWithFormat:@"Its a Draw!"];
    
    if (endingType == 1){
        CCLOG(@"Win");
        outputString = @"You Won!";
    } else if (endingType == 2) {
        CCLOG(@"Loss");
        outputString = @"You Lost!";
    } else {
        CCLOG(@"Draw");
    }
    
    
    // Setup Result Menu
    CCMenuItem *resultsOutput = [CCMenuItemFont itemFromString:(outputString) target:self selector:@selector(doNothing)];
    [(CCMenuItemFont *)resultsOutput setColor:ccBLACK];
    CCMenuItem *updateStats = [CCMenuItemFont itemFromString:@"You're stats have been updated!" target:self selector:@selector(doNothing)];
    [(CCMenuItemFont *)updateStats setColor:ccBLACK];
    CCMenuItem *backToMenu = [CCMenuItemFont itemFromString:@"Back To Menu" target:self selector:@selector(showMainMenu)];
    [(CCMenuItemFont *)backToMenu setColor:ccBLACK];
    
    resultsMenu = [CCMenu menuWithItems:resultsOutput,updateStats,backToMenu, nil];
    [resultsMenu alignItemsVerticallyWithPadding:screenSize.height * 0.059f];
    [resultsMenu setPosition:ccp(240,160)];
    [self addChild:resultsMenu z:1 tag:kResultsTagValue];
    
    [self updateStats]; 
}

// Accelerometer
-(void)accelerometer:(UIAccelerometer *)accelerometer didAccelerate:(UIAcceleration *)acceleration {
    int accelX = 0;
    int accelY = 0;
    int accelZ = 0;
    
    accelX = acceleration.x - ( (acceleration.x * kFilteringFactor) + (accelX * (1.0 - kFilteringFactor)) );
    accelY = acceleration.y - ( (acceleration.y * kFilteringFactor) + (accelY * (1.0 - kFilteringFactor)) );
    accelZ = acceleration.z - ( (acceleration.z * kFilteringFactor) + (accelZ * (1.0 - kFilteringFactor)) );
    
    if(accelX > 0.5 || accelY > 0.5){
        CCLOG(@"Motion Triggered");
        [self displayReadiedUp];
    }
}

#pragma mark -
#pragma mark Number Crunching

- (void)updateStats {
    // Update Stats in PLIST
     
    NSError *error;
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES); //1
    
    NSString *documentsDirectory = [paths objectAtIndex:0]; //2
    
    NSString *path = [documentsDirectory stringByAppendingPathComponent:@"Stats.plist"]; //3  
    
    NSFileManager *fileManager = [NSFileManager defaultManager];  
    
    if (![fileManager fileExistsAtPath: path]) //4
    {
        NSString *bundle = [[NSBundle mainBundle] pathForResource:(@"Stats") ofType:(@"plist")]; //5
        
        [fileManager copyItemAtPath:bundle toPath: path error:&error]; //6
    }
    
    NSMutableDictionary *savedStock = [[NSMutableDictionary alloc] initWithContentsOfFile: path];
    
    NSMutableDictionary *data = [[NSMutableDictionary alloc] initWithContentsOfFile: path];
    
    //load from savedStock example int value
    int matchesPlayed;
    int matchesWon;
    int matchesLost;
    
    matchesPlayed = [[savedStock objectForKey:@"MatchesPlayed"] intValue];
    matchesWon = [[savedStock objectForKey:@"MatchesWon"] intValue];
    matchesLost = [[savedStock objectForKey:@"MatchesLost"] intValue];
    
    [savedStock release];
    
    //here add elements to data file and write data to file
    matchesPlayed++;
    
    // Add 1 to games play 
    [data setObject:[NSNumber numberWithInt:matchesPlayed] forKey:@"MatchesPlayed"];
    
    if (endingType == 1) {
        matchesWon++;
        [data setObject:[NSNumber numberWithInt:matchesWon] forKey:@"MatchesWon"];
    } else {
        matchesLost++;
        [data setObject:[NSNumber numberWithInt:matchesLost] forKey:@"MatchesLost"];
    }
    [data writeToFile: path atomically:YES];
    [data release];
}

- (void) startGameplay {
    // generate player values
    [self generateValues];
    
    // Connect
    CCLOG(@"Run connect to start peer picker");
    [self connectToPeers:self];

}

- (void) calculateWinner {
    CCLOG(@"Calculating Winner");
    
    serverPlayerTotalValue = (serverPlayerAttack - clientPlayerDefence + randomValue + (20 - serverPlayerFame));
    
    clientPlayerTotalValue = (clientPlayerAttack - serverPlayerDefence + randomValue + (20 - clientPlayerFame));
    
    CCLOG(@"Server Total:%d", serverPlayerTotalValue);
    CCLOG(@"Client Total:%d", clientPlayerTotalValue);
    
    CCLOG(@"Sorting out the has Won Value");
    if (serverPlayerTotalValue > clientPlayerTotalValue){
        endingType = 1;
    }
    else if (serverPlayerTotalValue < clientPlayerTotalValue){
        endingType = 2;
    } 
    else {
        endingType = 3;
    }
    
    CCLOG(@"Won Value:%d", endingType);
    CCLOG(@"move to Display Results");
    [self displayResults];
}

- (void) generateValues {
    
    // load IDs from selected Plist and into an array
    NSError *error;
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES); //1
    NSString *documentsDirectory = [paths objectAtIndex:0]; //2
    NSString *cardSelectedPath = [documentsDirectory stringByAppendingPathComponent:@"SelectedCards.plist"]; //3
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    if (![fileManager fileExistsAtPath: cardSelectedPath]) //4
    {
        NSString *bundle = [[NSBundle mainBundle] pathForResource:(@"SelectedCards") ofType:(@"plist")]; //5
        
        [fileManager copyItemAtPath:bundle toPath: cardSelectedPath error:&error]; //6
    }
    
    NSMutableDictionary *cardSelectedData = [[NSMutableDictionary alloc] initWithContentsOfFile: cardSelectedPath];
    
    //load from savedStock
    int card1;
    int card2;
    int card3;
    
    card1 = [[cardSelectedData objectForKey:@"Card 1"] intValue];
    card2 = [[cardSelectedData objectForKey:@"Card 2"] intValue];
    card3 = [[cardSelectedData objectForKey:@"Card 3"] intValue];
    
    CCLOG(@"Card 1 = %d", card1);
    CCLOG(@"Card 2 = %d", card2);
    CCLOG(@"Card 3 = %d", card3);
    
    [cardSelectedData release];
    
    // use loaded values to load card data
    
    
    NSString *path = [[NSBundle mainBundle] bundlePath];
    NSString *finalPath = [path stringByAppendingPathComponent:@"CardData.plist"];
    NSDictionary *loadedCardData = [[NSDictionary dictionaryWithContentsOfFile:finalPath] retain];
    
    NSString *card1String = [NSString stringWithFormat:@"%d", card1];
    NSString *card2String = [NSString stringWithFormat:@"%d", card2];
    NSString *card3String = [NSString stringWithFormat:@"%d", card3];
    
    NSMutableArray *card1Data = [loadedCardData objectForKey:card1String];
    NSMutableArray *card2Data = [loadedCardData objectForKey:card2String];
    NSMutableArray *card3Data = [loadedCardData objectForKey:card3String];
    
    [loadedCardData release];
    
    int card1Attack = [[card1Data objectAtIndex:1]intValue];
    int card1Defence = [[card1Data objectAtIndex:2]intValue];
    int card1Fame = [[card1Data objectAtIndex:3]intValue];
    
    int card2Attack = [[card2Data objectAtIndex:1]intValue];
    int card2Defence = [[card2Data objectAtIndex:2]intValue];
    int card2Fame = [[card2Data objectAtIndex:3]intValue];
    
    int card3Attack = [[card3Data objectAtIndex:1] intValue];
    int card3Defence = [[card3Data objectAtIndex:2] intValue];
    int card3Fame = [[card3Data objectAtIndex:3] intValue];
    
    
    // from card data generate server attack, defence and remaining fame
    serverPlayerAttack = card1Attack + card2Attack + card3Attack;
    serverPlayerDefence = card1Defence + card2Defence +card3Defence;
    serverPlayerFame = card1Fame + card2Fame + card3Fame;
    
    CCLOG(@"Server Attack value is %d", serverPlayerAttack);
    CCLOG(@"Server Defence value is %d", serverPlayerDefence);
    CCLOG(@"Server Fame value is %d", serverPlayerFame);
    
    // Generate Random value
    serverPlayerRandomValue = ((rand() / RAND_MAX) * 20);
}

#pragma mark -
#pragma mark Networking Code

- (void) connectToPeers:(id) sender{
    CCLOG(@"Call Peer Picker");
    [peerPicker show];
}

- (void) sendPlayerData:(id)sender{
    CCLOG(@"Sending Player Data");
    
    NSMutableString *valuesMutableString = [NSMutableString stringWithFormat:@"%d", serverPlayerAttack];
    [valuesMutableString appendFormat:@":%d", serverPlayerDefence];
    [valuesMutableString appendFormat:@":%d", serverPlayerFame];
    [valuesMutableString appendFormat:@":%d", serverPlayerRandomValue];
    
    NSData *valueData = [valuesMutableString dataUsingEncoding:NSUTF8StringEncoding]; 
    
    // Send data to connected peer
    [gameSession sendData:valueData toPeers:sender withDataMode:GKSendDataReliable error:nil];    
}

#pragma mark -
#pragma mark GKPeerPickerControllerDelegate

// This creates a unique Connection Type for this particular applictaion
- (GKSession *)peerPickerController:(GKPeerPickerController *)picker sessionForConnectionType:(GKPeerPickerConnectionType)type{
    // Create a session with a unique session ID - displayName:nil = Takes the iPhone Name
    GKSession* session = [[GKSession alloc] initWithSessionID:@"K0014561.CowboyGame" displayName:nil sessionMode:GKSessionModePeer];
    return [session autorelease];
}

// Tells us that the peer was connected
- (void)peerPickerController:(GKPeerPickerController *)picker didConnectPeer:(NSString *)peerID toSession:(GKSession *)session{
    
    // Get the session and assign it locally
    self.gameSession = session;
    session.delegate = self;
    
    //No need of teh picekr anymore
    picker.delegate = nil;
    [picker dismiss];
    [picker autorelease];
}

// Function to receive data when sent from peer
- (void)receiveData:(NSData *)data fromPeer:(NSString *)peer inSession: (GKSession *)session context:(void *)context
{
    //Convert received NSData to NSString to display
    CCLOG(@"Receiving Data");
    NSString *clientData = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    NSArray *parsedClientData = [clientData componentsSeparatedByString:@":"];
    
    NSString *clientPlayerAttackString = [parsedClientData objectAtIndex:0];
    NSString *clientPlayerDefenceString = [parsedClientData objectAtIndex:1];
    NSString *clientPlayerFameString = [parsedClientData objectAtIndex:2];
    NSString *clientPlayerRandomString = [parsedClientData objectAtIndex:3];
    
    clientPlayerAttack = [clientPlayerAttackString intValue];
    clientPlayerDefence = [clientPlayerDefenceString intValue];
    clientPlayerFame = [clientPlayerFameString intValue];
    clientPlayerRandomValue = [clientPlayerRandomString intValue];
    
    randomValue = ((clientPlayerRandomValue+serverPlayerRandomValue)/2);
    
    CCLOG(@"Client Attack value is %d", clientPlayerAttack);
    CCLOG(@"Client Defence value is %d", clientPlayerDefence);
    CCLOG(@"Client Fame value is %d", clientPlayerFame);
    CCLOG(@"Random value is %d", randomValue);
    
    [gameSession release];
    CCLOG(@"Released Game Session");
    [self calculateWinner];
}

#pragma mark -
#pragma mark GKSessionDelegate

- (void)session:(GKSession *)session peer:(NSString *)peerID didChangeState:(GKPeerConnectionState)state{
    
    if(state == GKPeerStateConnected){
        // Add the peer to the Array
        [peerArray addObject:peerID];
        CCLOG(@"Peer Connected");

        // Used to acknowledge that we will be sending data
        [session setDataReceiveHandler:self withContext:nil];
        
        [self sendPlayerData:peerArray];

    }
}

@end
